import { ContainerOptions, PropertyOptions, SerdeOptions } from './options'
import { GlobalRegistry, Registry } from './registry'

function decorateContainer(target: any, context: any, options: ContainerOptions) {
  const meta = context.metadata
  const serde = (meta.serde || new SerdeOptions(target))
  serde.target = target
  serde.options = options
  meta.serde = serde
}

function decorateProperty(target: any, context: any, options: PropertyOptions) {
  const meta = context.metadata
  const serde = (meta.serde || new SerdeOptions(target))
  serde.properties.set(context.name, options)
  meta.serde = serde
}


export function serde(options: ContainerOptions | PropertyOptions) {
  return function(target: any, context: any) {
    if (target == null) {
      decorateProperty(target, context, options as PropertyOptions)
    } else
      decorateContainer(target, context, options as ContainerOptions)
  }
}

export function register(registry: Registry = GlobalRegistry) {
  return function(target: any, _context: any) {
    registry.add(target)
  }
}
